import json

import typer

from sss_cli import __version__
from sss_cli.helper import NoKeychainException, config, get_keychain


def load_key_from_chain(config_id: str) -> str:
    keychain = get_keychain()
    if not keychain.is_file():
        typer.secho("Keychain file not found, please run `sss key` first", fg="red")
        raise NoKeychainException
    with open(keychain, "r") as keychain_file:
        keychain_dict = json.load(keychain_file)
        if not keychain_dict["version"] == __version__:
            typer.secho(
                f"Keychain file version {keychain_dict['version']} and current version {__version__} do not match",
                fg="red",
            )
    try:
        key = keychain_dict.get("keys").get(config_id).get("key")
    except (AttributeError, KeyError):
        typer.secho(f"Keychain file does not contain `keys` or `{config_id}`", fg="red")
        raise NoKeychainException
    if not key:
        typer.secho(
            f"Keychain file does not contain key for config_id `{config_id}`", fg="red"
        )
        raise NoKeychainException
    return key


def get_real_key(key: str) -> str:
    if key == "":
        try:
            key = load_key_from_chain(config.config_id)
        except NoKeychainException:
            typer.secho("No keychain found, please run `sss key` first", fg="red")
            raise typer.Exit(code=1)

    return key



import requests
import typer


def fetch_encrypted(url):
    try:
        r = requests.get(url).text
        return r
    except:
        typer.secho(f"Error fetching {url}", fg="red")
        raise typer.Exit(code=1)



import typer

from sss_cli import __version__
from sss_cli._string_template import EXAMPLE_KEYCHAIN
from sss_cli.helper import get_keychain
from sss_cli.inject import inject
from sss_cli.share import share

app = typer.Typer()


@app.command("key")
def set_key(
    clear: bool = typer.Option(
        False, "-c", "--clear", help="Clear all keys in keychain"
    ),
    force: bool = typer.Option(
        False, "-f", "--force", help="Force clear all keys in keychain"
    ),
):
    """Edit keys in keychain."""
    keychain = get_keychain()
    if clear:
        if not force:
            typer.confirm("Are you sure you want to delete it?", abort=True)
        if keychain.is_file():
            keychain.unlink()
            typer.secho("Cleared keychain.", fg="green")
        return
    if not keychain.is_file():
        keychain.write_text(EXAMPLE_KEYCHAIN)
    typer.secho("Please edit keychain config file.", fg="green")
    typer.launch(str(keychain))


@app.command("share")
def cmd_share(
    config_path: str = typer.Argument(..., help="Path to your repo"),
    key: str = typer.Option("", "-k", "--key", help="Password as plaintext"),
):
    """Update the cypher file by encrypting the secret file."""
    share(config_path, key)


@app.command(name="inject")
def cmd_inject(
    config_path: str = typer.Argument(..., help="Path to your repo"),
    key: str = typer.Option("", "-k", "--key", help="Password as plaintext"),
):
    """Inject the decrypted cypher to correct path in repo."""
    inject(config_path, key)



from sss_cli import __version__
import json

EXAMPLE_KEYCHAIN_DICT = {
    "version": f"{__version__}",
    "keys": {
        "sss-example": {
            "key": "This key contains 32 characters.",
            "comment": "Some comment. This key is used for testing.",
        }
    },
}

EXAMPLE_KEYCHAIN = json.dumps(EXAMPLE_KEYCHAIN_DICT, indent=2)


def test_keychain_dict():
    print(EXAMPLE_KEYCHAIN_DICT)


if __name__ == "__main__":
    test_keychain_dict()



import codecs
import nacl.utils
import nacl.secret
import nacl.exceptions
import typer


def encrypt(message: str, key: str) -> str:
    """
    Encrypt string with key
    """
    try:
        box = nacl.secret.SecretBox(codecs.encode(key, "utf-8"))
    except nacl.exceptions.ValueError:
        typer.secho("The key must be exactly 32 bytes long ", fg="red")
        raise typer.Exit(code=1)
    encrypted = box.encrypt(codecs.encode(message, "utf-8"))
    b64 = codecs.encode(encrypted, "base64").decode("utf-8")
    return b64


def decrypt(encrypted: str, key: str) -> str:
    """
    Encrypt string with key
    """
    try:
        box = nacl.secret.SecretBox(codecs.encode(key, "utf-8"))
    except nacl.exceptions.ValueError:
        typer.secho("The key must be exactly 32 bytes long ", fg="red")
        raise typer.Exit(code=1)
    byte_msg = codecs.decode(bytes(encrypted, "utf-8"), "base64")
    try:
        decrypted = box.decrypt(byte_msg).decode("utf-8")
    except nacl.exceptions.CryptoError:
        typer.secho("Decryption failed. Ciphertext failed verification", fg="red")
        raise typer.Exit(code=1)
    return decrypted


def test():
    fake_key = "I'm a sentence of 32 characters."
    msg = "i'm a secret message, but not a sentence, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor, very very very long lorem ipsum dolor"

    encrypted = encrypt(msg, fake_key)
    decrypted = decrypt(encrypted, fake_key)

    print(f"encrypted: \n{encrypted}")
    print(f"decrypted: \n{decrypted}")


if __name__ == "__main__":
    test()



import enum
import json
from dataclasses import dataclass
from pathlib import Path

import typer

from sss_cli import APP_NAME, __version__

NOT_SET = str(enum.auto())


class NoKeychainException(Exception):
    pass


def get_keychain() -> Path:
    app_dir = Path(typer.get_app_dir(APP_NAME))
    app_dir.mkdir(parents=True, exist_ok=True)
    keychain: Path = app_dir / "keychain.json"
    return keychain


def write_file(file_path: Path, file_content: str) -> None:

    with open(file_path, "wb") as env_file:
        env_file.write(file_content.encode("utf-8"))
    typer.secho(f"Successfully write file to {file_path}", fg="green")


@dataclass(frozen=True)
class File_Map:
    source: str
    target: Path


class Config_Manager:
    def __init__(self) -> None:
        self.config_id: str = NOT_SET
        self.files: list[File_Map] = [File_Map(source=NOT_SET, target=Path())]
        self.key_identifier: str = NOT_SET
        self.last_key_rotation: int = 0
        self.sss_version: str = NOT_SET

    def load(self, config_path: Path) -> None:
        if not config_path.exists():
            typer.secho(f"Config file not found: {config_path}", fg="red")
            raise typer.Exit(code=1)
        folder = Path(config_path).parent
        with open(config_path, "r") as config_file:
            config_dict = json.loads(config_file.read())

        self.sss_version = config_dict["sss_version"]
        self.check_version()

        file_maps = []
        for file in config_dict["files"]:
            abs_target: Path = folder / file["target"]
            file_maps += [File_Map(file["source"], abs_target.absolute())]
        self.config_id = config_dict["config_id"]
        self.files = file_maps
        self.key_identifier = config_dict["key_identifier"]
        self.last_key_rotation = config_dict["last_key_rotation"]

    def check_version(self):
        if not self.sss_version == __version__:
            typer.secho(
                f"sss_cli version {__version__} and config version {self.sss_version} mismatch",
                fg="red",
            )
            raise typer.Exit(code=1)


config = Config_Manager()



from pathlib import Path

import typer

from sss_cli.keychain import get_real_key

from .encryption import decrypt
from .helper import config, write_file
from .remote import fetch_encrypted


def inject(
    config_path: str = typer.Argument(..., help="Path to your config_file"),
    key: str = typer.Option("", "-k", "--key", help="Password as plaintext"),
):
    """Inject the decrypted cypher to correct path in repo."""

    config.load(Path(config_path))
    # CLI options
    key = get_real_key(key)
    for file in config.files:
        cypher_string = fetch_encrypted(file.source)
        plain_secret = decrypt(cypher_string, key)
        write_file(file.target, plain_secret)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sss_cli']

package_data = \
{'': ['*']}

install_requires = \
['PyNaCl>=1.5.0,<2.0.0', 'requests>=2.27.1,<3.0.0', 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['sss = sss_cli.__main__:app',
                     'symmetric-secrete-share = sss_cli.__main__:app']}

setup_kwargs = {
    'name': 'symmetric-secrete-share',
    'version': '0.0.5',
    'description': 'Share secret files via github with symmetric encryption ed25519.',
    'long_description': '# symmetric-secrete-share\n\nCLI to share secret files via github with symmetric encryption ed25519.\n\n- **IMPORTANT: The secret files at should be git-ignored to avoid oblivious leakage.**\n- Temporarily supports only text files (only tested with `.env`).\n- Best used to store secrets and configurations.\n- Key should be a 32-byte long string.\n- (FAQ) If you share with GitHub, please notice that there\'s a 5 minutes cool-down on refreshing. [Details](https://stackoverflow.com/questions/46551413/github-not-update-raw-after-commit)\n\n## Use\n\n1. Install CLI `pip3 install symmetric-secrete-share`.\n2. Check the [Tutorial Chapter](#Tutorial) and `sss --help`.\n3. Recommended: set up a global key chain with `sss key`, or you would have to input key every time.\n4. Get a config like `$REPO_ROOT/test/injection/sss.json`. The json-schema inside will help you write it.\n\n### inject\n\n1. Get a config file like `$REPO_ROOT/test/injection/sss.json`. The json-schema inside will help you write it.\n2. run CLI\n\n   ```bash\n   sss inject [-k TEXT] CONFIG_PATH\n   ```\n\n### share\n\n1. Run CLI\n\n   ```bash\n   sss share [-k TEXT] CONFIG_PATH\n   ```\n\n2. Upload the generated file to GitHub (or other platforms).\n3. Update the config file if needed.\n\n## Contribute\n\n- Created for [Artcoin-Network](https://github.com/Artcoin-Network/), modifying the private repo[Artcoin-Network/artificial-dev-config](https://github.com/Artcoin-Network/artificial-dev-config).\n- To contribute, please fork the repo and run `poetry install`.\n- Read more in [dev-docs.md](./docs/dev-docs.md)\n\n## Tutorial\n\nIn this tutorial, all commands are assumed to run under the `$REPO_ROOT`. We are going to use these concepts:\n\n- key: `This key contains 32 characters.`.\n- URL: `https://raw.githubusercontent.com/PabloLION/symmetric-secrete-share/main/tests/example.encrypted`.\n- key chain: A file to share key, initialized with `sss key`.\n\nWe are going to play with the folder `test/injection`, with the `sss.json` file inside it. To share your own file, a new config file should be created.\n\n### Setup a local key chain\n\n```bash\nsss key # create/edit\nsss key -c # clear all keys\n```\n\n### load files from URL\n\nThese code will generate a `test/injection/target.env` like `test/example.env`\n\n```bash\nsss inject ./tests/injection/sss.json # use key from initial key chain\nsss inject -k "This key contains 32 characters." ./tests/injection/sss.json\nsss inject ./tests/injection/sss.json -k "I\'m a string with 32 characters." # fail\n```\n\n### share files\n\nNeed to upload manually #TODO\nThese code will generate a `test/injection/target.encrypted`\n\n```bash\nsss share ./tests/injection/sss.json # use key from initial key chain\nsss share -k "This key contains 32 characters." ./tests/injection/sss.json\n```\n',
    'author': 'Pablion',
    'author_email': '36828324+Pablion@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path

import typer

from sss_cli.keychain import get_real_key

from .encryption import encrypt
from .helper import config, write_file


def share(
    config_path: str = typer.Argument(..., help="Path to your repo"),
    key: str = typer.Option("", "-k", "--key", help="Password as plaintext"),
):
    """Update the cypher file by encrypting the secret file."""
    config.load(Path(config_path))
    key = get_real_key(key)
    for file in config.files:
        with open(file.target, "rb") as secret_file:
            secret_string = secret_file.read().decode("utf-8")
        write_file(file.target.with_suffix(".encrypted"), encrypt(secret_string, key))
        typer.secho("TODO: Manually upload and delete the encrypted file", fg="yellow")
        typer.secho("Encrypted file not uploaded/deleted", fg="red")
